<?php

namespace FiloBlu\Storelocator\ConfigProvider;

use Magento\Checkout\Model\ConfigProviderInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Api\StoreResolverInterface;

class CheckoutConfigProvider implements ConfigProviderInterface
{

    protected $_helper;
    protected $jsonHelper;
    protected $tagsFactory;
    protected $allowedCountryModel;
    protected $_checkoutSession;
    protected $storeFactory;

    public function __construct(
        \FiloBlu\Storelocator\Helper\Data $helper,
        \Magento\Framework\Webapi\Rest\Request\Deserializer\Json $jsonHelper,
        \FiloBlu\Storelocator\Model\TagsFactory $tagsFactory,
        \Magento\Directory\Model\AllowedCountries $allowedCountryModel,
        \Magento\Checkout\Model\Session $checkoutSession,
        \FiloBlu\Storelocator\Model\StoresFactory $storeFactory
    )
    {
        $this->_helper = $helper;
        $this->jsonHelper = $jsonHelper;
        $this->tagsFactory = $tagsFactory;
        $this->allowedCountryModel = $allowedCountryModel;
        $this->_checkoutSession = $checkoutSession;
        $this->storeFactory = $storeFactory;
    }

    /**
     * @return array
     */
    public function getConfig()
    {

        if (!$this->_helper->getConfig('carriers/pickup_in_store/active')) {
            return [
                'pickup_in_store' => [
                    'enabled' => false
                ]
            ];
        }

        $googleIncludePlaces = $this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_search_autocomplete_enabled') ? "&libraries=places" : "";
        $googleMapConfig = [
            'center' => [
                'lat' => (float)explode(";", $this->_helper->getConfig('filoblu_storelocator_section/map_options/gmaps_default_center'))[0],
                'lng' => (float)explode(";", $this->_helper->getConfig('filoblu_storelocator_section/map_options/gmaps_default_center'))[1]
            ],
            'zoom' => (int)$this->_helper->getConfig('filoblu_storelocator_section/map_options/gmaps_default_zoom'),
            'minZoom' => (int)$this->_helper->getConfig('filoblu_storelocator_section/map_options/gmaps_min_zoom'),
            'maxZoom' => (int)$this->_helper->getConfig('filoblu_storelocator_section/map_options/gmaps_max_zoom'),
            'mapId' => 'map',
            'mapTypeId' => $this->_helper->getConfig('filoblu_storelocator_section/map_options/gmaps_type'),
            'zoomControl' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/map_options/gmaps_ui_zoomControl_enabled'),
            'mapTypeControl' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/map_options/gmaps_ui_mapTypeControl_enabled'),
            'streetViewControl' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/map_options/gmaps_ui_streetViewControl_enabled'),
            'rotateControl' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/map_options/gmaps_ui_rotateControl_enabled'),
            'scaleControl' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/map_options/gmaps_ui_scaleControl_enabled'),
            'fullscreenControl' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/map_options/gmaps_ui_fullscreenControl_enabled'),
            'gmapApiUrl' => $this->_helper->getConfig('filoblu_storelocator_section/general/gmaps_api_url'),
            'gmapGeolocationApiUrl' => $this->_helper->getConfig('filoblu_storelocator_section/geolocation/gmaps_geolocation_api_url'),
            'gmapApiKey' => $this->_helper->getConfig('filoblu_storelocator_section/general/gmaps_api_key'),
            'gmapLanguage' => explode("_", \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Framework\Locale\Resolver')->getLocale())[0],
            'gmapGeoLocation' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/geolocation/gmaps_geolocation_enabled'),
            'gmapGeoLocationZoom' => (int)$this->_helper->getConfig('filoblu_storelocator_section/geolocation/gmaps_geolocation_zoom'),
            'gmapGeoLocationMarker' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/geolocation/gmaps_geolocation_marker_enabled'),
            'gmapGeoLocationMarkerAnimation' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/geolocation/gmaps_geolocation_marker_animation_enabled'),
            'gmapGeoLocationMarkerImage' => $this->_helper->getConfig('filoblu_storelocator_section/geolocation/gmaps_geolocation_marker_image') ? $this->_helper->getBaseUrlConfigImage() . $this->_helper->getConfig('filoblu_storelocator_section/geolocation/gmaps_geolocation_marker_image') : "",
            'gmapGeoLocationDistance' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/geolocation/gmaps_geolocation_distance_enabled'),
            'gmapGeoLocationDuration' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/geolocation/gmaps_geolocation_duration_enabled'),
            'gmapGeoLocationTravelMode' => $this->_helper->getConfig('filoblu_storelocator_section/geolocation/gmaps_geolocation_travelmode'),
            'gmapGeoLocationUnitSystem' => (int)$this->_helper->getConfig('filoblu_storelocator_section/geolocation/gmaps_geolocation_unitsystem'),
            'gmapGeoLocationGetDirections' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/geolocation/gmaps_geolocation_getdirections_enabled'),
            'gmapGeoLocationGetDirectionsPanelId' => 'gmapGetDirectionsPanel',
            'gmapMarkersMarkerImage' => $this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_marker_image') ? $this->_helper->getBaseUrlConfigImage() . $this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_marker_image') : "",
            'gmapStoresImageDir' => $this->_helper->getBaseUrlStoresImage(),
            'gmapMarkersClusterMarker' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_marker_enabled'),
            'gmapMarkersClusterMarkerGridSize' => (int)$this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_marker_gridSize'),
            'gmapMarkersClusterMarkerMaxZoom' => $this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_marker_maxZoom') ? $this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_marker_maxZoom') : null,
            'gmapMarkersClusterMarkerZoomOnClick' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_marker_zoomOnClick'),
            'gmapMarkersClusterMarkerAverageCenter' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_marker_averageCenter'),
            'gmapMarkersClusterMarkerMinimumClusterSize' => (int)$this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_minimumClusterSize'),
            'gmapMarkersClusterMarkerImage' => $this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_image') ? $this->_helper->getBaseUrlConfigImage() . $this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_image') : $this->_helper->getViewFileUrl('FiloBlu_Storelocator::images/cluster_marker.png'),
            'gmapMarkersClusterMarkerImageWidth' => (int)$this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_image_width'),
            'gmapMarkersClusterMarkerImageHeight' => (int)$this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_image_height'),
            'gmapMarkersClusterMarkerTextColor' => $this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_textColor'),
            'gmapMarkersClusterMarkerTextSize' => (int)$this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_textSize'),
            'gmapMarkersClusterMarkerTextOffsetWidth' => (int)$this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_text_offsetWidth'),
            'gmapMarkersClusterMarkerTextOffsetHeight' => (int)$this->_helper->getConfig('filoblu_storelocator_section/markers/gmaps_markers_cluster_text_offsetHeight'),
            'gmapSearchAutocomplete' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_search_autocomplete_enabled'),
            'gmapSearchAutocompleteId' => 'gmapSearch',
            'gmapSearchAutocompleteType' => $this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_search_autocomplete_type') ? $this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_search_autocomplete_type') : "",
            'gmapSearchAutocompleteZoom' => (int)$this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_search_autocomplete_zoom'),
            'gmapSearchCountryId' => 'gmapCountry',
            'gmapSearchCityId' => 'gmapCity',
            'gmapSearchCountry' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_search_country_enabled'),
            'gmapSearchCity' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_search_city_enabled'),
            'gmapSearchTags' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_search_tag_enabled'),
            'gmapSearchTagsContainerClass' => 'tag-info',
            'gmapStoresListId' => 'gmapStoresList',
            'gmapWorkingTimes' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/workingtimes/gmaps_working_times_enabled'),
            'baseUrl' => $this->_helper->getBaseUrl(),
            'pickup_in_store' => 1,
            'allowedCountries' => $this->allowedCountryModel->getAllowedCountries()
        ];

        if ($styleMapConfig = $this->_helper->getConfig('filoblu_storelocator_section/map_options/gmaps_style')) {
            $googleMapConfig["styles"] = $this->jsonHelper->deserialize(str_replace("styles:", "", $styleMapConfig));
        }

        $allAvailableTagHtml = "";

        foreach ($this->tagsFactory->create()->getAllAvailableTags() as $tag) {
            $allAvailableTagHtml .= '<div class="tag-info" data-tag-id="' . $tag->getTagId() . '">';

            if ($tag->getImageUrl()) {

                $allAvailableTagHtml .= '<div class="tag-image"><img src="' . $tag->getImageUrl() . '" title="' . strip_tags($tag->getDescription()) . '" alt="' . $tag->getName() . '" /></div>';
            }

            $allAvailableTagHtml .= '<div class="tag-name">' . $tag->getName() . '</div></div>';
        }

        // Current Customer Store Data
        $quote = $this->_checkoutSession->getQuote();

        $currentSelectedStoreId = 0;
        $currentSelectedStoreData = [];

        if ($quote) {
            $currentSelectedStoreId = ($quote->getPickupInStoreId() ?: 0);
            $model = $this->storeFactory->create()->load($currentSelectedStoreId);
            if ($model->getId()) {
                $currentSelectedStoreData = $model->getData();
                $region = "";
                if ($currentSelectedStoreData["region"]) {
                    $region = ', ' . $currentSelectedStoreData["region"];
                }
                $currentSelectedStoreData["address"] = '</br>' . $currentSelectedStoreData["street"] . '</br>' . $currentSelectedStoreData["postcode"] . ' ' . $currentSelectedStoreData["city"] . $region . ' (' . $currentSelectedStoreData["country"] . ')';
            }
        }

        return [
            'pickup_in_store' => [
                'enabled' => true,
                'current_selected_store_data' => $currentSelectedStoreData,
                'google_map_script_src' => $this->_helper->getConfig('filoblu_storelocator_section/general/gmaps_api_url') . '?language=' . explode("_", \Magento\Framework\App\ObjectManager::getInstance()->get('Magento\Framework\Locale\Resolver')->getLocale())[0] . '&key=' . $this->_helper->getConfig('filoblu_storelocator_section/general/gmaps_api_key') . $googleIncludePlaces,
                'google_map_config' => $googleMapConfig,
                'gmaps_search_country_enabled' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_search_country_enabled'),
                'gmaps_search_city_enabled' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_search_city_enabled'),
                'gmaps_search_autocomplete_enabled' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_search_autocomplete_enabled'),
                'gmaps_search_tag_enabled' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_search_tag_enabled'),
                'gmaps_geolocation_enabled' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_geolocation_enabled'),
                'gmaps_geolocation_getdirections_enabled' => (bool)$this->_helper->getConfig('filoblu_storelocator_section/search/gmaps_geolocation_getdirections_enabled'),
                'gmaps_all_available_tags' => $allAvailableTagHtml
            ]
        ];
    }

}
